unit Com_Uart;

interface

uses Comm;

const
 BufferSize     = 8196;
 BufferMax      = 8196;

type
  PUart_Object = ^TUart_Object;
  TUart_Object = object(TComm)
    constructor Init(ComPort:Byte; FlowState:Byte);
    destructor Done; virtual;
    procedure Setup(Baud: longint; DataBits: byte; Parity: Char; Stop: byte);  virtual;
    procedure Dtr(State: boolean); virtual;
    function  Carrier: boolean; virtual;
    procedure ClearOut; virtual;
    procedure ClearIn; virtual;
    procedure FlushOut; virtual;
    function  Received: boolean; virtual;
    function  Receive: char; virtual;
    procedure Send(character: char); virtual;
    function  Ring: boolean; virtual;
    procedure HangUp; virtual;
    procedure Flow(State: byte); virtual;
    function  TxReady : boolean; virtual;
    procedure SetBaud(Baud: LongInt);  virtual;
    procedure PutBlock(var Buf; Size: Word);
    function  GetBaud: LongInt; virtual;
    private
     Old_IER, Old_IIR, Old_LCR, Old_MCR, Old_IMR: Byte;
     ExitSave, OriginalVector:  Pointer;
     Fifos, PortThere : Boolean;
     {
     Base: Word;
     IRQ: Byte;
     Buffer: array [0..BufferMax] of byte ;
     BufferHead: Word;
     BufferTail: Word;
     OverFlow: boolean;
     Pause: boolean;
     }
  end;


implementation
uses Dos, Timer;

const
 MaxPhysPort          = 7;
 I8088_IMR            = $21 ; { port address of the Interrupt Mask Register }
 IBM_UART_THR         = $00 ;
 IBM_UART_RBR         = $00 ;
 IBM_UART_IER         = $01 ; { Interrupt register}
 IBM_UART_IIR         = $02 ; { Interrupt Idintifer }
 IBM_UART_LCR         = $03 ; { Drive register }
 IBM_UART_MCR         = $04 ; { Modem Drive rigister }
 IBM_UART_LSR         = $05 ; { status register}
 IBM_UART_MSR         = $06 ; { modem status }
 PortTable  : array [0..MaxPhysPort] of record
   Base : Word;
   IRQ  : Byte
 end = ( (Base : $3F8;  IRQ : 4),
         (Base : $2F8;  IRQ : 3),
         (Base : $3E8;  IRQ : 4),
         (Base : $2E8;  IRQ : 3),
         (Base : 0;  IRQ : 0),
         (Base : 0;  IRQ : 0),
         (Base : 0;  IRQ : 0),
         (Base : 0;  IRQ : 0));


const
  IsOpen: array [0..MaxPhysPort] of boolean = (false, false, false, false,
                                               false, false, false, false);

procedure DisableInterrupts;  Inline ($FA);
procedure EnableInterrupts;   Inline ($FB);

var
 Base: Word;
 IRQ: Byte;
 Buffer: array [0..BufferMax] of byte ;
 BufferHead: Word;
 BufferTail: Word;
 Overflow: boolean;
 Pause: boolean;
{!!!!!!!!!!!!!!!!!}

const
 XON  = 17;
 XOFF = 19;

procedure ISR;  Interrupt; assembler;
asm
 sti
@Start:
    { get the incoming character                                        }
    { Buffer[BufferHead] := chr(port[base + ibm_uart_rbr]);             }
 mov dx,Base
 in al,dx

 cmp al, XON
 je  @XONLbl
 cmp al, XOFF
 je  @XOFFLbl
 jmp @Begin

@XONLbl:
 mov Pause, False
 jmp @L003

@XOFFLbl:
 mov Pause,True
 jmp @L003

@Begin:
 mov Pause, False
 mov bx,BufferHead
 mov Buffer[bx].Byte,al
    { BufferNewHead := Succ (BufferHead);                               }
 inc bx
    { if BufferNewHead > BufferMax then BufferNewHead := 0 ;            }
 cmp bx,BufferMax
 jle @L001
 xor bx,bx
    { if BufferNewHead = BufferTail then Overflow := true               }
@L001:
 cmp bx,BufferTail
 jne @L002
 mov overflow,1
 jmp @L003
    { ELSE BEGIN                                                        }
    {   BufferHead := BufferNewHead;                                    }
    {   Async_BufferUsed := succ(Async_BufferUsed);                     }
    {   IF Async_BufferUsed > Async_MaxBufferUsed then                  }
    {     Async_MaxBufferUsed := Async_BufferUsed                       }
    {   END ;                                                           }
@L002:
 mov BufferHead,bx
 Add dx,5
    { Check FIFO - And process if more bytes.                           }
 In al,dx
 And al,$01
 cmp al,$01
 je @start
@L003:
 cli
    { issue non-specific EOI                                            }
    { port[$20] := $20 ;                                                }
 mov al,20h
 out 20h,al
end;

constructor TUart_Object.Init(Comport:Byte; FlowState:byte);
begin
 if not IsOpen[Pred (Comport)] then
   begin
     BufferHead := 0;
     BufferTail := 0;
     Overflow := False;
     UsedPort := Pred (Comport);
     Fifos := False;
     IsOpen[UsedPort] := False;
     if PortTable [UsedPort].Base <> 0 then
     begin
       Base := PortTable [UsedPort].Base;
       DTR (False);
       {Delay(100);}
       IRQ := PortTable [UsedPort].IRQ;
       Old_IER := Port [IBM_UART_IER + Base];
       Old_MCR := Port [IBM_UART_MCR + Base];
       Old_LCR := Port [IBM_UART_LCR + Base];
       Port [IBM_UART_LCR + Base] := $75;
       PortThere := (Port [IBM_UART_LCR + Base] = $75);
       Port [IBM_UART_LCR + Base] := $3;
    end;
   end;
end;

destructor TUart_Object.Done;
begin
 if IsOpen[UsedPort] then
 begin
   DisableInterrupts;
   Port [I8088_IMR] := (Port[I8088_IMR] OR (1 SHL IRQ));
   Port [IBM_UART_IER + Base] := Old_IER;
   EnableInterrupts;
   Port [IBM_UART_MCR + Base] := Old_MCR;
   Port [IBM_UART_LCR + Base] := Old_LCR;
   SetIntVec (IRQ + 8, OriginalVector);
   IsOpen[UsedPort] := False;
 end;
 inherited Done;
end;

procedure TUart_Object.Setup(Baud: longint; DataBits: byte; Parity: Char; Stop: byte);
 (*
 procedure Comm_SetDirect (NewRate : LongInt);
 var
  I, J, K : Word;
  Temp    : LongInt;
  Parity_byte: byte;
 begin
  case Parity of
     'N': Parity_byte := 0;
     'O': Parity_byte := 1;
     'X': Parity_byte := 2;
     'E': Parity_byte := 3;
      else Parity_byte := 0;
   end;
   Temp := 115200;
   Temp := Temp div Newrate;
   Move (Temp, J, 2);
   K := Port [IBM_UART_LCR + Base];
   Port [IBM_UART_LCR + Base] := $80;
   Port [IBM_UART_THR + Base] := Lo (J);
   Port [IBM_UART_IER + Base] := Hi (J);
   Port [IBM_UART_LCR] := (DataBits - 5) or (Stop - 1) * 4 or Parity_byte * 8;
   Port[IBM_UART_LCR + Base] := $0B;
{   Port [IBM_UART_LCR + Base] := 3;}
 end;
  *)
 var
  I, OldIIR        : Byte;
begin
 inherited Setup(Baud, DataBits, Parity, Stop);
 if PortThere then
   begin
     SetBaud(Baud);
     Port [IBM_UART_MCR + Base] := $0B;
     OldIIR := Port [IBM_UART_IIR + Base];
     Port [IBM_UART_IIR + Base] := 1;
     Fifos := (Port [IBM_UART_IIR + Base] AND $C0 = $C0);
     if not Fifos then Port [IBM_UART_IIR + Base] := OldIIR;
     GetIntVec (IRQ + 8, OriginalVector);
     DisableInterrupts;
     SetIntVec (IRQ + 8, @ISR);
     Port[IBM_UART_MCR + Base] := $08; {- Enable OUT2 on port -}
     I:=Port [IBM_UART_LSR + Base];
     I:=Port [IBM_UART_MSR + Base];
     I:=Port [IBM_UART_RBR + Base];
     Port[IBM_UART_IER + Base] := $01;  {- Set port data avail.int. -}
     Port [I8088_IMR] := (Port [I8088_IMR] AND ((1 SHL IRQ) XOR $FF));
     EnableInterrupts;
     IsOpen[UsedPort] := True;
     DTR (True);
   end;
 {Async_Open := IsOpen[UsedPort];}
end;

procedure TUart_Object.DTR(State : Boolean);
var
 I : Byte;
 CPort: Word;
begin
 if State then
   Port [IBM_UART_MCR + Base] := $0B else
   Port [IBM_Uart_MCR +Base] := $0A;
end;

procedure TUart_Object.Flow(State: byte);
begin

end;

function TUart_Object.Carrier: boolean;
begin
 Carrier := Port [IBM_UART_MSR + Base] AND $80 = $80;
end;

procedure TUart_Object.ClearIn;
begin
 while Received do Receive;
 BufferHead := 0;
 BufferTail := 0;
 OverFlow   := False;
end;

procedure TUart_Object.ClearOut;
begin

end;

procedure TUart_Object.FlushOut;
begin

end;

function TUart_Object.TxReady : boolean;
var
 Ahigh          : byte;
 Carr, CTS, THR : boolean;
 CTS_RTS_On: boolean;
begin
  inherited TxReady;
  CTS_RTS_On:=True;
  THR := ((Port [IBM_UART_LSR + Base] and $20) <> 0);
  CTS :=  (Port [IBM_UART_MSR + Base] and $10 = $10);
  if CTS_RTS_On and Carrier then
      TxReady := THR and Cts else
      TxReady :=THR;
end;

function TUart_Object.Received: boolean;
begin
 inherited Received;
 DisableInterrupts;
 Received := (Bufferhead <> BufferTail);
 EnableInterrupts;
end;

function TUart_Object.Receive: char;
begin
 inherited Receive;
 if Received then
 begin
  Receive:=Chr(Buffer [BufferTail]);
  BufferTail := (Succ (BufferTail) mod BufferSize);
 end else
 Receive:=#0;
end;

procedure TUart_Object.Send(character: char);
 const
  MaxWait = 30;
 var
  T : TTimer;
begin
 inherited Send(character);

 T.Init(MaxWait);
 repeat until TxReady or T.TimeOut;
 T.Done;

 T.Init(MaxWait);
 repeat until (not Pause) or T.TimeOut;
 T.Done;

 if not TxReady then Exit;

 Port [IBM_UART_THR + Base] := Ord (Character);
end;

function TUart_Object.Ring: boolean;
begin
  case UsedPort of
    0 : Ring := boolean (Port[$3FE] and 64);
    1 : Ring := boolean (Port[$2FE] and 64);
    2 : Ring := boolean (Port[$3EE] and 64);
    3 : Ring := boolean (Port[$2EE] and 64);
     else Ring := False;
  end;
end;

procedure TUart_Object.HangUp;
var
 Tcount: Integer;
begin
 Dtr(False);
 Delay(600);
 Dtr(True);
 if Carrier = True then
  begin
    Tcount := 1;
    repeat
      Send('+');
      Send('+');
      Send('+');
      Send(Char(13));
      Delay(3000);
      Send('A');
      Send('T');
      Send('H');
      Send('0');
      Send(Char(13));
      Delay(3000);
      if Carrier = False
        then
          tcount := 3;
      Tcount := Tcount + 1;
    until Tcount = 4;
  end;
end;

procedure TUart_Object.SetBaud(Baud: LongInt);
var I, J, K : Word;
    Temp    : LongInt;
begin
  Temp := 115200;
  Temp := Temp div Baud;
  Move (Temp, J, 2);
  K := Port [IBM_UART_LCR + Base];
  port [IBM_UART_LCR + Base] := $80;
  Port [IBM_UART_THR + Base] := Lo (J);
  Port [IBM_UART_IER + Base] := Hi (J);
  Port [IBM_UART_LCR + Base] := 3;
end;

function TUart_Object.GetBaud: LongInt;
var
 I, J, K : Word;
 Temp    : LongInt;
begin
  K := Port [ibm_UART_LCR + Base];
  Port [IBM_UART_LCR + Base] := K OR $80;
  i := Port [IBM_UART_THR + Base];
  J := Port [IBM_UART_IER + Base];
  J := J * $100;
  J := J + I;
  Port [IBM_UART_LCR + base] := k;
  Temp := 115200;
  Temp := Temp DIV J;
  GetBaud := Temp;
  (*
  Port [IBM_UART_LCR + Base] :=  Port [IBM_UART_LCR + Base] or $80;
  K:=PortW[IBM_UART_THR + Base];
  Port [IBM_UART_LCR + Base] :=  Port [IBM_UART_LCR + Base] and $7F;
  if (K <> 0) then
    GetBaud:=longint(115200 div K) else
    GetBaud:=75;
  *)
end;

procedure TUart_Object.PutBlock(var Buf; Size: Word);
type
 TBuf = array [0..$FFF0] of char;
var
 I: Word;
begin
 for i:=0 to Size-1 do
  begin
   Send(TBuf(Buf)[i]);
  end;
end;

end.



var
 I, OldIIR        : Byte;
 PortThere : Boolean;

(*  Async_Open(14400, ComPort); *)
  if not IsOpen[Pred(ComPort)] then
  begin
    BufferHead := 0;
    BufferTail := 0;
    Overflow := False;
    UsedPort := Pred (ComPort);
    IsOpen[UsedPort] := False;
    if PortTable [UsedPort].Base <> 0 Then
    begin
      Base := PortTable [UsedPort].Base;
      IRQ := PortTable [UsedPort].IRQ;
      Old_IER := Port [IBM_UART_IER + Base];
      Old_MCR := Port [IBM_UART_MCR + Base];
      Old_LCR := Port [IBM_UART_LCR + Base];
      Port [IBM_UART_LCR + Base] := $75;
      PortThere := (Port [IBM_UART_LCR + Base] = $75);
      Port [IBM_UART_LCR + Base] := $3;
      DTR (False);
   end;
  end else
  begin
   Fail;
  end;
 if not PortThere then ;
